package org.opencdmp.deposit.controller;

import gr.cite.tools.auditing.AuditService;
import gr.cite.tools.logging.LoggerService;
import gr.cite.tools.logging.MapLogEntry;
import org.opencdmp.commonmodels.models.dmp.DmpModel;
import org.opencdmp.deposit.zenodorepository.audit.AuditableAction;
import org.opencdmp.depositbase.repository.DepositConfiguration;
import org.opencdmp.deposit.zenodorepository.service.zenodo.ZenodoDepositService;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.AbstractMap;
import java.util.Map;

@RestController
@RequestMapping("/api/deposit")
public class DepositController implements org.opencdmp.depositbase.repository.DepositController {
    private static final LoggerService logger = new LoggerService(LoggerFactory.getLogger(DepositController.class));

    private final ZenodoDepositService depositClient;

    private final AuditService auditService;

    @Autowired
    public DepositController(ZenodoDepositService depositClient, AuditService auditService) {
        this.depositClient = depositClient;
	    this.auditService = auditService;
    }

    public String deposit(@RequestBody DmpModel dmpModel, @RequestParam("authToken")String authToken) throws Exception {
        logger.debug(new MapLogEntry("deposit " + DmpModel.class.getSimpleName()).And("dmpModel", dmpModel));

        String doiId = depositClient.deposit(dmpModel, authToken);
        
        this.auditService.track(AuditableAction.Deposit_Deposit, Map.ofEntries(
                new AbstractMap.SimpleEntry<String, Object>("dmpModel", dmpModel)
        ));
        return doiId;
    }

    public String authenticate(@RequestParam("authToken") String code) {
        logger.debug(new MapLogEntry("authenticate " + DmpModel.class.getSimpleName()));

        String token = depositClient.authenticate(code);

        this.auditService.track(AuditableAction.Deposit_Authenticate);
        
        return token;
    }

    public DepositConfiguration getConfiguration() {
        logger.debug(new MapLogEntry("getConfiguration " + DmpModel.class.getSimpleName()));
        
        DepositConfiguration configuration = depositClient.getConfiguration();
        
        this.auditService.track(AuditableAction.Deposit_GetConfiguration);
        
        return configuration;
    }

    public String getLogo() {
        logger.debug(new MapLogEntry("getLogo " + DmpModel.class.getSimpleName()));

        String logo = depositClient.getLogo();

        this.auditService.track(AuditableAction.Deposit_GetLogo);

        return logo;
    }

}
